## $Id: browser.py 11637 2014-05-13 10:37:46Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import hashlib
import grok
from zope.interface import Interface
from zope.component import queryAdapter
from kofacustom.nigeria.interswitch.browser import (
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPageStudent
    )
from waeup.aaua.students.interfaces import ICustomStudentOnlinePayment
from waeup.aaua.applicants.interfaces import ICustomApplicantOnlinePayment
from waeup.aaua.interfaces import MessageFactory as _

PRODUCT_ID = '105'
SITE_NAME = 'aaua.waeup.org'
PROVIDER_ACCT = '6012015294'
PROVIDER_BANK_ID = '117'
PROVIDER_ITEM_NAME = 'BT Education'
INSTITUTION_NAME = 'AAUA'
CURRENCY = '566'
GATEWAY_AMT = 150.0
#QUERY_URL = 'https://webpay.interswitchng.com/paydirect/services/TransactionQueryURL.aspx'
#QUERY_URL = 'https://testwebpay.interswitchng.com/test_paydirect/services/TransactionQueryURL.aspx'

POST_ACTION = 'https://webpay.interswitchng.com/paydirect/webpay/pay.aspx'
#POST_ACTION = 'https://testwebpay.interswitchng.com/test_paydirect/webpay/pay.aspx'

HOST = 'webpay.interswitchng.com'
#HOST = 'testwebpay.interswitchng.com'

URL = '/paydirect/services/TransactionQueryWs.asmx'
#URL = '/test_paydirect/services/TransactionQueryWs.asmx'
httplib.HTTPConnection.debuglevel = 0

def interswitch_img_url(view):
    static = view.static
    if static is None or static.get(
        'interswitch_verve_mastercard.gif', None) is None:
        static = queryAdapter(
            view.request, Interface, name='waeup.aaua.interswitch')
    return static['interswitch_verve_mastercard.gif']()

class CustomInterswitchPageStudent(InterswitchPageStudent):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    grok.template('student_goto_interswitch')
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID
    #mac = ''

    def interswitch_img_url(self):
        return interswitch_img_url(self)

    def update(self):
        student, certificate, xmldict = super(
            CustomInterswitchPageStudent, self).update()
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        # Institution data
        xmldict['institution_acct'] = "0000000000000"
        xmldict['institution_bank_id'] = '0'
        xmldict['institution_item_name'] = self.category
        xmldict['institution_name'] = INSTITUTION_NAME
        self.pay_item_id = '000'
        if self.context.p_category in ('schoolfee', 'schoolfee_1', 'schoolfee_2'):
            provider_amt = 3000.0
            if self.context.p_category == 'schoolfee_2':
                provider_amt = 0.0
            xmldict['provider_amt'] = 100 * provider_amt
            self.pay_item_id = '10500'
            xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - provider_amt -
                GATEWAY_AMT)
            if student.current_mode.endswith('_pt'):
                xmldict['institution_acct'] = "0321100000000046"
                xmldict['institution_bank_id'] = "89"
            elif student.current_mode.endswith('_sw'):
                xmldict['institution_acct'] = "2461770000021"
                xmldict['institution_bank_id'] = "120"

        #hashargs = (
        #    self.context.p_id +
        #    PRODUCT_ID +
        #    self.pay_item_id +
        #    str(int(self.amount_auth)) +
        #    self.site_redirect_url +
        #    self.mac)
        #self.hashvalue = hashlib.sha512(hashargs).hexdigest()

        # Interswitch amount is not part of the xml data

        if self.context.p_category in ('schoolfee', 'schoolfee_1'):
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        elif self.context.p_category  == 'schoolfee_2':
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict
        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        return

class CustomInterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL